/****************************************************************************************/
/* For constructing 3D worlds.								*/
/*											*/
/* Compile with:                                                                	*/
/*  cc move_objects_over_socket.c -lm -o move_objects_over_socket
/*                                                                              	*/
/****************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "soc_lib3WF.c"  /* From www.atl.lmco.com/proj/xgraph/soclib.tar */

int soc;
char ofile[2000];
float *landalt;

/* Assumes objects stored as:
/*
	<def_obj> type_name
	 <tri> <col r g b t/>  <vrt t1x t1y t1z />  <col r g b /> <vrt t2x t2y t2z />  <col r g b />  <vrt t3x t3y t3z /> </tri>
	 <col r g b t/>  
	 <tri> <vrt t1x t1y t1z />  <vrt t2x t2y t2z />  <vrt t3x t3y t3z /> </tri>
	  ...
	</obj>

	<inst_obj> instance_name type_name
         x y z  xAxisAngle yAxisAngle zAxisAngle
	</inst_obj> 

	...
  Where x,y,z is initial positon.
  Ie. In def_obj, color can be anywhere or (almost) nowhere.
  Color r,g,b and t, where t is transparency (1=solid, 0=clear, 0.5=translucent, etc.).
  
*/

float RCONST = 1.0/2147483647.0;	/* set up the rand normalizing constant */

float duration=200.0;


int vertcnt=0;
float vrtmem[3][3];
float old_color[4]={-1.0,0.0,0.0,0.0};

int color_compare( float *new, float *old )
{ int j;
  if ((new[0]==old[0]) && (new[1]==old[1]) && (new[2]==old[2]) && (new[3]==old[3])) return 1;
  else { old[0] = new[0];  old[1] = new[1]; old[2] = new[2]; old[3] = new[3]; return 0; }
}


int mysendsoc( int sockfd, char *line )
{
 printf("Sending: '%s'\n", line );
 return send_soc( sockfd, line );
}


void sendvertex( float *colorv, float x,  float y, float z )
{
 int i;
 float n1[3], n2[3], dotpr, vert[3];
	
 vert[0] = x; vert[1] = y; vert[2] = z;
 for (i=0; i<3; i++) vrtmem[vertcnt][i] = vert[i];
 if (vertcnt==2) 
  { 
   sprintf(ofile,"<tri>");   send_soc( soc, ofile );

   if (!color_compare(colorv,old_color))
    { sprintf(ofile,"<col r=\"%g\" g=\"%g\" b=\"%g\" t=\"%g\"/>", colorv[0], colorv[1], colorv[2], colorv[3] ); send_soc( soc, ofile ); }
   for (vertcnt=0; vertcnt<3; vertcnt++) 
     { sprintf(ofile,"<vrt x=\"%g\" y=\"%g\" z=\"%g\"/>", vrtmem[vertcnt][0], vrtmem[vertcnt][1], vrtmem[vertcnt][2] );  send_soc( soc, ofile ); }

   sprintf(ofile,"</tri>\n");   send_soc( soc, ofile ); 
   vertcnt = 0;
  }
 else vertcnt++;
}



float landcolor[4],  watercolor[4];

void cavg( float *c1, float cA, float cB, float *c4 )
{
 int i;
 float *c2, *c3;

 if (cA>0.0) c2 = landcolor; else c2 = watercolor;
 if (cB>0.0) c3 = landcolor; else c3 = watercolor;
 for (i=0; i<3; i++)
  c4[i] = 0.3333 * (c1[i] + c2[i] + c3[i]);
}


void subdivide( float x, float dx, float z, float dz, 
		float *color1, float *color2, float *color3, float *color4, 
		float alt1, float alt3, float alt2, float alt4,
		int N )
{
 float *cmat;
 int i, j, k, m;

 /* Original dimension, N=2.  One extra point (5-extra in 2D), N=3.  Assume N = 4 or 5.  Etc.. */
 cmat = (float *)malloc( (N+1) * (N+1) * 5 * sizeof(float) );
 for (i=0; i<N; i++)
  for (j=0; j<N; j++)
   {
    for (k=0; k<3; k++)
     cmat[(i*N+j)*5+k] = 0.2;
    cmat[(i*N+j)*5+3] = 1.0;
    cmat[(i*N+j)*5+4] = 0.0;
   }

 /* Initialize the outer boundaries.  (On which all else depends.) */
 for (i=0; i<N; i++)
  {
   for (k=0; k<3; k++)
    {
     cmat[(i)*5+k] = ((float)(N-i-1)/(float)(N-1)) * color1[k] + ((float)i/(float)(N-1)) * color2[k];
     cmat[(i*N)*5+k] = ((float)(N-i-1)/(float)(N-1)) * color1[k] + ((float)i/(float)(N-1)) * color3[k];
     cmat[((N-1)*N+i)*5+k] = ((float)(N-i-1)/(float)(N-1)) * color3[k] + ((float)i/(float)(N-1)) * color4[k];
     cmat[(i*N+(N-1))*5+k] = ((float)(N-i-1)/(float)(N-1)) * color2[k] + ((float)i/(float)(N-1)) * color4[k];
    }
   cmat[(i)*5+4] = ((float)(N-i-1)/(float)(N-1)) * alt1 + ((float)i/(float)(N-1)) * alt2;
   cmat[(i*N)*5+4] = ((float)(N-i-1)/(float)(N-1)) * alt1 + ((float)i/(float)(N-1)) * alt3;
   cmat[((N-1)*N+i)*5+4] = ((float)(N-i-1)/(float)(N-1)) * alt3 + ((float)i/(float)(N-1)) * alt4;
   cmat[(i*N+(N-1))*5+4] = ((float)(N-i-1)/(float)(N-1)) * alt2 + ((float)i/(float)(N-1)) * alt4;
  } 

 /* Low-pass it, interpolate. */
 for (m=0; m<N; m++)
  {
   for (i=1; i<N-1; i++)
    for (j=1; j<N-1; j++)
     {
      for (k=0; k<3; k++)
       cmat[(i*N+j)*5+k] = 0.15 * (cmat[((i-1)*N+j)*5+k] + cmat[(i*N+j-1)*5+k] + cmat[((i+1)*N+j)*5+k] + cmat[(i*N+j+1)*5+k]) +
			  0.075 * (cmat[((i-1)*N+j-1)*5+k] + cmat[((i-1)*N+j+1)*5+k] + cmat[((i+1)*N+j-1)*5+k] + cmat[((i+1)*N+j+1)*5+k]);
      cmat[(i*N+j)*5+4] = 0.15 * (cmat[((i-1)*N+j)*5+4] + cmat[(i*N+j-1)*5+4] + cmat[((i+1)*N+j)*5+4] + cmat[(i*N+j+1)*5+4]) +
			  0.075 * (cmat[((i-1)*N+j-1)*5+4] + cmat[((i-1)*N+j+1)*5+4] + cmat[((i+1)*N+j-1)*5+4] + cmat[((i+1)*N+j+1)*5+4]);
     }

  }
 
 dx = dx / (float)(N-1);
 dz = dz / (float)(N-1);

 /* Put it out. */
 for (i=0; i<N-1; i++)
  for (j=0; j<N-1; j++)
   {
    float tc[4];

    tc[3] = 1.0;
    for (k=0; k!=3; k++) tc[k] = 0.333 * (cmat[(j*N+i)*5+k] + cmat[((j+1)*N+i+1)*5+k] + cmat[(j*N+i+1)*5+k]);
    sendvertex( tc, x+(float)i*dx, cmat[(i*N+j)*5+4], z+(float)j*dz );
    i = i + 1;  j = j + 1;
    sendvertex( tc, x+(float)i*dx, cmat[(i*N+j)*5+4], z+(float)j*dz );
    j = j - 1;
    sendvertex( tc, x+(float)i*dx, cmat[(i*N+j)*5+4], z+(float)j*dz );  i = i - 1;

    for (k=0; k!=3; k++) tc[k] = 0.333 * (cmat[(j*N+i)*5+k] + cmat[((j+1)*N+i+1)*5+k] + cmat[((j+1)*N+i)*5+k]);
    sendvertex( tc, x+(float)i*dx, cmat[(i*N+j)*5+4], z+(float)j*dz );
    j = j + 1;
    sendvertex( tc, x+(float)i*dx, cmat[(i*N+j)*5+4], z+(float)j*dz );  i = i + 1;
    sendvertex( tc, x+(float)i*dx, cmat[(i*N+j)*5+4], z+(float)j*dz );  j = j - 1;  i = i - 1;
   }
 free(cmat);
}



float horizon_cusp( float x, float z, float xmin, float xmax, float zmin, float zmax )
{
 float y1=0.0, y2=0.0, y, y3=0.0, y4=0.0, y5=0.0;

    if (xmax-x < 1800.0)  y1 = 350.0 - (xmax-x) * (350.0/1800); else
    if (x-xmin < 1800.0)  y1 = 350.0 - (x-xmin) * (350.0/1800);
    if (zmax-z < 1800.0)  y2 = 350.0 - (zmax-z) * (350.0/1800); else
    if (z-zmin < 1800.0)  y2 = 350.0 - (z-zmin) * (350.0/1800); 
    if (y1>y2) y = y1; else y = y2;

    if (xmax-x < 300.0)  y3 = 350.0 - (xmax-x-300.0) * (350.0/600); else
    if (x-xmin < 300.0)  y3 = 350.0 - (x-xmin-300.0) * (350.0/600);
    if (zmax-z < 300.0)  y4 = 350.0 - (zmax-z-300.0) * (350.0/600); else
    if (z-zmin < 300.0)  y4 = 350.0 - (z-zmin-300.0) * (350.0/600); 
    if (y3>y4) y5 = y3; else y5 = y4;
    if (y5>0.0) y = y5;

 y = y + (50.0 + y) * ((float)random() * RCONST);

 return y;
}




int tank_cnt=0;

void Tank( float length, float width, float height, float r, float g, float b,
		float posx, float posz, float ang )
{
 float x, y, z, dx, dz, xmin, ymin, zmin, xmax, zmax, ymax, colorV[4], xmaxtop, xmintop;
 int i;

 sprintf(ofile,"<inst_obj name=\"Tank_%d\" kind=\"TankT\" x=\"%g\" y=\"50\" z=\"%g\" xang=\"0\" yang=\"%g\" zang=\"0\" />\n", tank_cnt++, posx, posz, ang);
 send_soc( soc, ofile );
}





int jet_cnt=0;

void Jet( float width, float length, float height, float altitude, float r, float g, float b,
		float posx, float posz, float ang )
{
 float x, y, z, dx, dz, xmin, ymin, zmin, xmax, zmax, ymax, colorV[4];

 sprintf(ofile,"<inst_obj name=\"Jet_%d\" kind=\"JetT\" x=\"%g\" y=\"%g\" z=\"%g\" xang=\"0\" yang=\"%g\" zang=\"0\" />\n", jet_cnt++, posx, altitude, posz, ang);
 send_soc( soc, ofile ); 
}



int helo_cnt=0;

void Helo( float width, float length, float height, float altitude, float r, float g, float b,
		float posx, float posz, float ang )
{
 float x, y, z, dx, dz, xmin, ymin, zmin, xmax, zmax, ymax, colorV[4];

 sprintf(ofile,"<inst_obj name=\"helo_%d\" kind=\"HeloT\" x=\"%g\" y=\"%g\" z=\"%g\" xang=\"0\" yang=\"%g\" zang=\"0\" />\n", helo_cnt, posx, altitude, posz, ang);
 send_soc( soc, ofile ); 

 ymax = 1.2 * height;  
 xmin = - 0.8 * length;  xmax = 0.8 * length;
 zmin = - 0.05 * length;  zmax = 0.05 * length;
 sprintf(ofile,"<inst_obj name=\"Blade_%d\" kind=\"Blade\" x=\"%g\" y=\"%g\" z=\"%g\" xang=\"0\" yang=\"%g\" zang=\"0\" />\n", helo_cnt++, posx, altitude, posz, ang);
 send_soc( soc, ofile ); 
 sprintf(ofile,"<ro obj=\"Blade_%d\" T1=\"0\" T2=\"%g\" xang=\"0\" yang=\"%g\" zang=\"0\"/>\n", helo_cnt-1, duration, 360 * duration);
 send_soc( soc, ofile ); 
}





/********************************************************************************************************/
/* OrbitCenter - This routine calculates an object's center of orbit, given it's initial position,	*/
/* a radius of orbit, and a second position on (the tangent) of it's trajectory.			*/
/* Also, of course, a switch to find the 'port' (-1) or 'starboard' (+1) side center.			*/
/* Basically, this calculates the x-y coord of a point 90-degrees to the right or left of the object at */
/* the distance of the radius.  Result returned in first two params.					*/
/********************************************************************************************************/
void orbitcenter( float *centx, float *centy, float radius, int side, float x1, float y1, float x2, float y2 )
{
 float dx, dy, h;

 dx = x1 - x2;  dy = y1 - y2;
 h = radius / sqrt(dx*dx + dy*dy);

 *centx = x1 + (float)side * dy * h;
 *centy = y1 - (float)side * dx * h;
}


/********************************************************************************************************/
/* Center_Orbit - This routine calculates an object's center of orbit, given it's initial position,	*/
/* and angle of orientation, assumed as the angle of the tangent of orbit at that point.		*/
/* Also, of course, a switch to find the 'port' (-1) or 'starboard' (+1) side center.			*/
/* Basically, this calculates the x-y coord of a point 90-degrees to the right or left of the object at */
/* the distance of the radius.  Launches orbit commands.						*/
/********************************************************************************************************/
void center_orbit( float radius, int side, float x, float z, float ang)
{
 float centx, centy;

 centx = x - radius * (float)side * sin(-3.14159 * ang/180.0);
 centy = z + radius * (float)side * cos(-3.14159 * ang/180.0);
 sprintf(ofile,"<orbity obj=\"Tank_%d\" T1=\"0\" T2=\"200\"  xcntr=\"%g\"  ycntr=\"%g\" zcntr=\"%g\"  ang=\"%g\"/>\n", tank_cnt-1, centx, 50.0, centy, (float)side * 20.0 * duration );
 send_soc( soc, ofile ); 
 sprintf(ofile,"<ro obj=\"Tank_%d\" T1=\"0\" T2=\"200\"  xang=\"0\" yang=\"%g\" zang=\"0\"/>\n", tank_cnt-1, - (float)side * 20.0 * duration + ang);
 send_soc( soc, ofile ); 
}




/*================================================================*/

int main( int argc, char **argv )
{
  int i, side=1, massive=0;
  int socno=13330;
  float Tnow=0.0, T1, T2;


  i = 1;
  while (argc>i)
   {
    if (strcmp(argv[i],"-soc")==0)
     {
      i++;  if (i==argc) {printf("Error: Missing socket number on commandline\n"); exit(0);}
      if (sscanf(argv[i],"%d",&socno)!=1) printf("Error: Bad socket number '%s' on commandline\n", argv[i]);
      else printf("Set socno to %d\n", socno);
     }
    else
    if (strcmp(argv[i],"-massive")==0)
     {
	massive = 1;	printf("Massive-fast population growth.\n");
     }
    else printf("Unknown command line argument '%s\n", argv[i]);
    i++;
   }

  printf("Server opening socket %d\n", socno); 
  soc = makeConnect( "localhost", socno );
  printf("Server made socket %d\n",socno);



  /* initialize the random generator seed: */
  srandom( 123 );  

 /*---------------------------------------------------------------*/
 /* Describe any object models.  Object ID's should begin at "1". */
 /*---------------------------------------------------------------*/


system("sleep 1");
 /* Tank( length, width, height, r, g, b, posx, posz, ang ) */
 Tank( 40.0, 25.0, 12.0, 0.2, 0.4, 0.3, -800.0, -2300.0, -270.0 );
 system("sleep 1");
 center_orbit( 200.0, side, -800.0, -2300.0, -270.0 );

 Tank( 40.0, 25.0, 12.0, 0.2, 0.4, 0.3, -460.0, -2280.0, 50.0 );
 usleep(100000);
 Tank( 40.0, 25.0, 12.0, 0.2, 0.4, 0.3, -200.0, -2400.0, 100.0 );
 usleep(100000);
 Tank( 40.0, 25.0, 12.0, 0.2, 0.4, 0.3, -20.0, -2300.0, -30.0 );
 usleep(100000);
 Tank( 40.0, 25.0, 12.0, 0.2, 0.4, 0.3, 100.0, -2600.0, -60.0 );
 usleep(100000);
 Tank( 40.0, 25.0, 12.0, 0.2, 0.4, 0.3, 20.0, -2500.0, 30.0 );
 usleep(100000);
 Tank( 40.0, 25.0, 12.0, 0.2, 0.4, 0.3, 200.0, -2700.0, 10.0 );
 usleep(100000);
 Tank( 40.0, 25.0, 12.0, 0.2, 0.4, 0.3, -180.0, -2900.0, 300.0 );
 usleep(100000);
 Tank( 40.0, 25.0, 12.0, 0.2, 0.4, 0.3, -120.0, -3100.0, -80.0 );

 for (i=0; i<15; i++)
  {
   float x, z, ang;

   usleep(100000);

   x = - 1400.0 * ((float)random() * RCONST) - 1600.0;
   z = 2000.0 * ((float)random() * RCONST) + 1200.0;
   ang = 360.0 * ((float)random() * RCONST);
   if (ang>200.0)
    Tank( 40.0, 25.0, 12.0, 0.04, 0.2, 0.12, x, z, ang );
   else
    Tank( 40.0, 25.0, 12.0, 0.2, 0.4, 0.3, x, z, ang );

   usleep(100000);
   center_orbit( 200.0, side, x, z, ang);  side = -side;
  }


 /* Jet( width, length, height, altitude, r, g, b,       posx,    posz,  ang ); */
 Jet(    40.0,  80.0,   25.0,   50.0,    0.5, 0.9, 0.85, -1200.0, 700.0, 60.0 );
 usleep(100000);
 Jet(    40.0,  80.0,   25.0,   50.0,    0.5, 0.9, 0.85, -1125.0, 800.0, 60.0 );
 usleep(100000);
 Jet(    40.0,  80.0,   25.0,   50.0,    0.5, 0.9, 0.85, -1050.0, 900.0, 60.0 );

 for (i=0; i<8; i++)
 {
  float centx, centz;
  int side;

  usleep(100000);
  Jet( 40.0, 60.0, 20.0, 450.0 + (float)i + 50.0, 0.5, 0.9, 0.85,  (float)i * 220.0, (float)i * 120.0, -30.0 );

  if (i % 2 == 0) side = -1; else side = 1;

  orbitcenter( &centx, &centz,  2000.0, side, (float)i * 220.0, (float)i * 120.0, (float)(i-1) * 220.0, (float)(i-1) * 120.0 );
  sprintf(ofile,"<orbity obj=\"Jet_%d\" T1=\"5\" T2=\"2000\"  xcntr=\"%g\"  ycntr=\"%g\"  zcntr=\"%g\"  ang=\"%g\"/>\n", jet_cnt-1, centx, 350.0 + (float)i + 50.0, centz, (float)side * 18000.0 );
  send_soc( soc, ofile ); 
  sprintf(ofile,"<ro obj=\"Jet_%d\" T1=\"5\" T2=\"2000\"  xang=\"0\" yang=\"%g\" zang=\"0\"/>\n", jet_cnt-1, - (float)side * 18000.0 );
  send_soc( soc, ofile ); 
 }

 Helo( 40.0, 80.0, 25.0, 180.0, 0.0, 0.015, 0.005, -1000.0, -300.0, -30.0 );
 usleep(100000);
 Helo( 40.0, 80.0, 25.0, 250.0, 0.0, 0.1, 0.015, -1200.0, -1200.0, 60.0 );



 for (i=0; i<150; i++)
  {
   float x, z, ang;

   usleep(50000);
   x = -6000.0 * ((float)random() * RCONST) + 2300.0;
   z = 3000.0 * ((float)random() * RCONST);
   ang = 360.0 * ((float)random() * RCONST);
   if (ang>200.0)
    Tank( 40.0, 25.0, 12.0, 0.04, 0.2, 0.12, x, z, ang );
   else
    Tank( 40.0, 25.0, 12.0, 0.2, 0.4, 0.3, x, z, ang );

  if (tank_cnt % 10 == 0)
   { char tmpstr[1000];

     send_soc( soc, "<time />\n");	/* Ask what time it is? */
     send_soc( soc, "<nop />\n");
     if (recv_soc( soc, tmpstr, 1000 ) < 0)  printf("Move_objects: socket error\n");
     if (sscanf( &(tmpstr[3]), "%f", &Tnow )!=1) printf("Move_objects: error reading time (%s)\n", tmpstr);;
     printf("T = %g\n", Tnow );

     sprintf(tmpstr,"<beam  obj1=\"Tank_%d\" obj2=\"Tank_%d\" t1=\"%f\" t2=\"%f\" r=\"1.0\" g=\"0.2\" b=\"0.2\" />\n", tank_cnt-1, tank_cnt - 2, Tnow+10.0, Tnow+15.0 );
     send_soc( soc, tmpstr);
   }

   usleep(10000);
   center_orbit( 200.0, side, x, z, ang);  side = -side;
  }

 for (i=0; i<8; i++)
 {
  float centx, centz;
  int side;

  usleep(10000);
  Jet( 40.0, 60.0, 20.0, 550.0 + (float)i + 50.0, 0.5, 0.9, 0.85,  (float)i * 220.0, (float)i * 120.0, -30.0 );

  if (i % 2 == 0) side = -1; else side = 1;

  orbitcenter( &centx, &centz,  2000.0, side, (float)i * 220.0, (float)i * 120.0, (float)(i-1) * 220.0, (float)(i-1) * 120.0 );
  sprintf(ofile,"<orbity obj=\"Jet_%d\" T1=\"20\" T2=\"2200\"  xcntr=\"%g\"  ycntr=\"%g\"  zcntr=\"%g\"  ang=\"%g\"/>\n", jet_cnt-1, centx, 350.0 + (float)i + 50.0, centz, (float)side * 18000.0 );
  send_soc( soc, ofile ); 
  sprintf(ofile,"<ro obj=\"Jet_%d\" T1=\"20\" T2=\"2200\"  xang=\"0\" yang=\"%g\" zang=\"0\"/>\n", jet_cnt-1, - (float)side * 18000.0 );
  send_soc( soc, ofile ); 
 }

 for (i=0; i<4500; i++)
  {
   float x, z, ang;

   if (massive) usleep(50000);
   else usleep(500000);
   x = -5000.0 * ((float)random() * RCONST) + 2500.0;
   z = 4500.0 * ((float)random() * RCONST) - 1000.0;
   ang = 360.0 * ((float)random() * RCONST);
   if (ang>200.0)
    Tank( 40.0, 25.0, 12.0, 0.04, 0.2, 0.12, x, z, ang );
   else
    Tank( 40.0, 25.0, 12.0, 0.2, 0.4, 0.3, x, z, ang );

if (tank_cnt % 5 == 0)
 { char tmpstr[1000];

   send_soc( soc, "<time />\n");	/* Ask what time it is? */
   send_soc( soc, "<nop />\n");
   if (recv_soc( soc, tmpstr, 1000 ) < 0)  printf("Move_objects: socket error\n");
   if (sscanf( &(tmpstr[3]), "%f", &Tnow )!=1) printf("Move_objects: error reading time (%s)\n", tmpstr);;

   sprintf(tmpstr,"<beam  obj1=\"Tank_%d\" obj2=\"Tank_%d\" t1=\"%f\" t2=\"%f\" r=\"1.0\" g=\"0.2\" b=\"0.2\" />\n", tank_cnt-1, tank_cnt - 2, Tnow+10.0, Tnow+15.0 );
   send_soc( soc, tmpstr);
 }

   usleep(1000);
   center_orbit( 200.0, side, x, z, ang);  side = -side;
   if (tank_cnt % 10 == 0) printf("Tank %d\n", tank_cnt);
  }


 system("sleep 30");
 printf("SERVER CLOSING SOCKET\n");
 system("sleep 1");
 send_soc( soc, "EndOfSocket");
 system("sleep 40");
 printf(" (ending socket now)\n");
 // close_soc( soc );
 close( soc );
 return 0;
}
